@@ -36,7 +36,7 @@
 #include <glib.h>
 #include <gtk/gtk.h>
 #include <libintl.h>
-//#include <conic.h>
+#include <libexif/exif-data.h>
 
 #define HOME_DIR g_get_home_dir()
 
@@ -341,6 +341,21 @@ gboolean personal_photo_frame_timeout (PersonalPhotoFrame *self)
 	return self->priv->isShown;
 }
 
+gint personal_photo_frame_exif_orientation (gchar *orientation)
+{
+	gint retVal=0;
+	if (g_str_equal (orientation, "top - left")) {
+		retVal = 0;
+	} else if (g_str_equal (orientation, "bottom - right")) {
+		retVal = 180;
+	} else if (g_str_equal (orientation, "right - top")) {
+		retVal = 270;
+	} else if (g_str_equal (orientation, "left - bottom")) {
+		retVal = 90;
+	}
+	return retVal;
+}
+
 void personal_photo_frame_update_content (PersonalPhotoFrame *self)
 {
 	//g_warning("%s", __PRETTY_FUNCTION__);
@@ -352,6 +367,18 @@ void personal_photo_frame_update_content (PersonalPhotoFrame *self)
 		
 		GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale (self->priv->singleFilename, self->priv->actualSize - 10, self->priv->actualSize - 10, TRUE, NULL);
 		if (pixbuf) {
+			ExifData *ed = exif_data_new_from_file (self->priv->singleFilename);
+			if (ed) {
+				ExifEntry *entry = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_ORIENTATION);
+				if (entry) {
+					char buf[1024];
+					exif_entry_get_value (entry, buf, sizeof(buf));
+					
+					gint rotation = personal_photo_frame_exif_orientation (buf);
+					pixbuf = gdk_pixbuf_rotate_simple (pixbuf, rotation);
+				}
+			}
+			exif_data_unref(ed);
 			g_object_unref (self->priv->image);
 			self->priv->image = gdk_pixbuf_copy (pixbuf);
 			g_object_unref (pixbuf);
@@ -384,6 +411,18 @@ void personal_photo_frame_update_content (PersonalPhotoFrame *self)
 
 					GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file_at_scale (self->priv->slideshowFilename, self->priv->actualSize - 10, self->priv->actualSize - 10, TRUE, NULL);
 					if (pixbuf) {
+						ExifData *ed = exif_data_new_from_file (self->priv->slideshowFilename);
+						if (ed) {
+							ExifEntry *entry = exif_content_get_entry (ed->ifd[EXIF_IFD_0], EXIF_TAG_ORIENTATION);
+							if (entry) {
+								char buf[1024];
+								exif_entry_get_value (entry, buf, sizeof(buf));
+								
+								gint rotation = personal_photo_frame_exif_orientation (buf);
+								pixbuf = gdk_pixbuf_rotate_simple (pixbuf, rotation);
+							}
+						}
+						exif_data_unref(ed);
 						g_object_unref (self->priv->image);
 						self->priv->image = gdk_pixbuf_copy (pixbuf);
 						g_object_unref (pixbuf);
@@ -707,14 +746,14 @@ static gboolean personal_photo_frame_expose_event (GtkWidget *widget, GdkEventEx
 		gdk_cairo_set_source_pixbuf (cr, self->priv->backButton, 5, height - 53);
 		cairo_paint_with_alpha (cr, 0.5);
 		
-		gdk_cairo_set_source_pixbuf (cr, self->priv->forwardButton, width - 53, height - 53);
+		gdk_cairo_set_source_pixbuf (cr, self->priv->forwardButton, width - 58, height - 53);
 		cairo_paint_with_alpha (cr, 0.5);
 		
-		cairo_move_to (cr, 5, height - 53);
-		cairo_line_to (cr, 5, height - 5);
-		cairo_line_to (cr, 5 + 48, height - 5);
-		cairo_line_to (cr, 5 + 48, height - 53);
-		cairo_line_to (cr, 5, height - 53);
+		cairo_move_to (cr, 0, height - 58);
+		cairo_line_to (cr, 0, height);
+		cairo_line_to (cr, 58, height);
+		cairo_line_to (cr, 58, height - 58);
+		cairo_line_to (cr, 0, height - 58);
 		
 		if (self->priv->leftArrowPressed) {
 			gtk_style_lookup_color (gtk_rc_get_style(widget), "SelectionColor", &color);
@@ -727,11 +766,11 @@ static gboolean personal_photo_frame_expose_event (GtkWidget *widget, GdkEventEx
 		cairo_set_line_width (cr, 1);
 		cairo_stroke (cr);
 		
-		cairo_move_to (cr, width - 53, height - 53);
-		cairo_line_to (cr, width - 53, height - 5);
-		cairo_line_to (cr, width - 5, height - 5);
-		cairo_line_to (cr, width - 5, height - 53);
-		cairo_line_to (cr, width - 53, height - 53);
+		cairo_move_to (cr, width - 58, height - 58);
+		cairo_line_to (cr, width - 58, height);
+		cairo_line_to (cr, width, height);
+		cairo_line_to (cr, width, height - 58);
+		cairo_line_to (cr, width - 58, height - 58);
 		
 		if (self->priv->rightArrowPressed) {
 			gtk_style_lookup_color (gtk_rc_get_style(widget), "SelectionColor", &color);
